import pypelinez.utilities as utilities


def current_branch():
    """Current git branch name

    Returns:
    str:Returning current git branch name

    """

    result = utilities.action("git rev-parse --abbrev-ref HEAD")
    utilities.assert_result(result)
    return utilities.transform_result(result)


def create_branch(name):
    result = utilities.action("git checkout -b " + name)
    utilities.print_result(result)
    utilities.assert_result(result)


def change_to_branch(name):
    result = utilities.action("git checkout " + name)
    utilities.print_result(result)
    utilities.assert_result(result)


def add():
    result = utilities.action("git add -A")
    utilities.print_result(result)
    utilities.assert_result(result)


def commit():
    result = utilities.action("git commit -F COMMIT.md")
    utilities.print_result(result)
    utilities.assert_result(result)


def merge(branch):
    result = utilities.action("git merge --squash " + branch)
    utilities.print_result(result)
    utilities.assert_result(result)


def delete(branch):
    result = utilities.action("git branch -D " + branch)
    utilities.print_result(result)
    utilities.assert_result(result)


def pull(source, branch):
    result = utilities.action("git pull " + source + " " + branch)
    utilities.print_result(result)
    utilities.assert_result(result)


# merge request
# https://docs.gitlab.com/ee/user/project/merge_requests/merge_when_pipeline_succeeds.html
def merge_request(target, description):
    current = current_branch()

    request = """\
    git push \\
    -o merge_request.create \\
    -o merge_request.target={target} \\
    -o merge_request.merge_when_pipeline_succeeds \\
    -o merge_request.remove_source_branch \\
    -o merge_request.title=\"Merge request for {current}\" \\
    -o merge_request.description=\"{description}\" \\
    origin {current}
  """.format(
        target=target, current=current, description=description
    )

    result = utilities.action(request)
    utilities.print_result(result)
    utilities.assert_result(result)


# merge request
# https://docs.gitlab.com/ee/user/project/merge_requests/merge_when_pipeline_succeeds.html
# def release_request(version):

#     branch = current_branch()

#     request = """\
#     git push \\
#     -o merge_request.create \\
#     -o merge_request.target=develop \\
#     -o merge_request.merge_when_pipeline_succeeds \\
#     -o merge_request.remove_source_branch \\
#     -o merge_request.title=\"Merge request for Release {version}\" \\
#     -o merge_request.description=\"Release {version}\" \\
#     origin {branch}
#   """.format(
#         version=version, branch=branch
#     )

#     result = utilities.action(request)
#     utilities.print_result(result)
#     utilities.assert_result(result)


def get_current_local_tag():
    result = utilities.action("git describe --tags --abbrev=0")
    utilities.assert_result(result)
    return str(result.stdout.decode("utf-8")).strip()


def get_current_remote_tags():
    result = utilities.action("git ls-remote --tags --sort=committerdate")
    utilities.assert_result(result)

    remote_versions = list(str(result.stdout.decode("utf-8")).split("\n"))

    if len(remote_versions) == 0:

        return []

    else:

        remote_versions = [i.strip() for i in remote_versions if i]

        remote_versions = list(map(lambda x: utilities.get_semantic_version(x), remote_versions))

        remote_versions = [i for i in remote_versions if i]

        remote_versions = [str(i.group(1)) for i in remote_versions if i]

        return list(reversed(remote_versions))


def create_tag(version):
    result = utilities.action("git tag -a " + version + " -m " + version)
    utilities.print_result(result)
    utilities.assert_result(result)



import click
import os
import pypelinez.utilities as utilities
import pypelinez.git as git
import toml


def get_config():
    cwd = os.getcwd()
    file = open(os.path.join(cwd, "pyproject.toml"), "r", encoding="utf8")
    config = toml.loads(file.read())
    file.close()
    return config


# group
@click.group()
def python():
    pass


@python.group()
def version():
    pass


@version.command()
def get():
    result = utilities.action("poetry version -s", os.getcwd())
    utilities.print_result(result)
    utilities.assert_result(result)


# config = get_config()
# version = Version(config['tool']['poetry']['version'])
# print(version)


@version.command()
@click.argument(
    "value",
    type=click.Choice(
        ["major", "minor", "patch", "premajor", "preminor", "prepatch", "prerelease"], case_sensitive=False
    ),
)
def bump(value):
    def bumped(value):
        switch = {
            "major": "poetry version major",
            "minor": "poetry version minor",
            "patch": "poetry version patch",
            "premajor": "poetry version premajor",
            "preminor": "poetry version preminor",
            "prepatch": "poetry version prepatch",
            "prerelease": "poetry version prerelease",
        }
        return switch.get(value, "Invalid bump expectation")

    result = utilities.action(bumped(value), os.getcwd())
    utilities.print_result(result)
    utilities.assert_result(result)


@version.command()
def tag():
    current_branch = git.current_branch()
    if current_branch == "main":
        result = utilities.action("poetry version -s", os.getcwd())
        utilities.assert_result(result)
        git.tag(result)
    else:
        print("Must tag on main branch")



#!/usr/bin/env python3

import click
from pypelinez.feature import feature
from pypelinez.release import release
from pypelinez.publish import publish
from pypelinez.apple import apple
from pypelinez.python import python


@click.group()
@click.version_option()
def main():
    pass


main.add_command(feature)
main.add_command(release)
main.add_command(publish)
main.add_command(apple)
main.add_command(python)

if __name__ == "__main__":
    main()



import click
import pypelinez.git as git
from pypelinez.click_ext import assertUsage


# -------------------------- commands

# group
@click.group()
def feature():
    pass


# name
@feature.command()
def name():
    print(git.current_branch())


# start
@feature.command()
@click.argument("name")
def start(name):
    """Creates a new branch with the format feature/<name>


    Parameters:
    name (str): feature branch name

    """

    assertUsage(git.current_branch() == "develop", "Must create feature from develop branch")

    click.echo("Creating feature: feature/" + name)

    git.create_branch("feature/" + name)


# commit
@feature.command()
def add_commit():

    git.add()

    git.commit()


@feature.command()
def submit():

    merge_request_file = open("MERGE_REQUEST.md", "r")
    merge_description = merge_request_file.read()
    merge_request_file.close()

    git.merge_request("develop", merge_description)


# finish
@feature.command()
def finish():
    branch = git.current_branch()

    git.change_to_branch("develop")

    git.pull("origin", "develop")

    git.delete(branch)



import os
import subprocess
import sys
import shutil
import re

current = os.getcwd()

# -------------------------- utilities


def action(command, cwd=current):
    return subprocess.run(command, cwd=cwd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)


def assert_result(result):
    if result.returncode != 0:
        sys.exit(result.returncode)


def transform_result(result):
    # https://stackoverflow.com/questions/41918836/how-do-i-get-rid-of-the-b-prefix-in-a-string-in-python
    return (result.stdout.decode("UTF-8") + result.stderr.decode("UTF-8")).strip()


def print_result(result):
    print(transform_result(result))


def remove_directory(directory):
    shutil.rmtree(directory)


def validate_semantic_version(version):
    version_regex = r"^(0|[1-9]\d*)\.(0|[1-9]\d*)\.(0|[1-9]\d*)(?:-((?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*)(?:\.(?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*))*))?(?:\+([0-9a-zA-Z-]+(?:\.[0-9a-zA-Z-]+)*))?$"  # noqa: E501
    pattern = re.compile(version_regex)
    match = pattern.match(version)
    return match is not None


def get_semantic_version(text):
    version_regex = r"^.*((0|[1-9]\d*)\.(0|[1-9]\d*)\.(0|[1-9]\d*)(?:-((?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*)(?:\.(?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*))*))?(?:\+([0-9a-zA-Z-]+(?:\.[0-9a-zA-Z-]+)*))?)$"  # noqa: E501
    pattern = re.compile(version_regex)
    match = pattern.match(text)
    return match



import click
import pypelinez.git as git
from pypelinez.click_ext import assertUsage, assertBadParameter, assertLocalVersionIsLatest
import pypelinez.utilities as utilities
from semantic_version import Version

# -------------------------- commands


# group
@click.group()
def release():
    pass


# start
@release.command()
@click.argument("version")
def start(version):
    """Creates a new release branch"""

    assertBadParameter(utilities.validate_semantic_version(version), "Version must follow Semantic Versioning rules")

    assertUsage(git.current_branch() == "develop", "Must create feature from develop branch")

    click.echo("Creating release: release/" + version)

    git.create_branch("release/" + version)


# commit
@release.command()
def add_commit():

    git.add()

    git.commit()


# submit release
@release.command()
def submit():
    """Submits the release as a merge request"""

    current, _, version = validate_release_branch()

    assertBadParameter(utilities.validate_semantic_version(version), "Version must follow Semantic Versioning rules")

    click.echo("Submitting " + current + " for release")

    merge_description = "Release " + version + " submission"

    git.merge_request("develop", merge_description)


# finish
@release.command()
def finish():
    current, _, version = validate_release_branch()

    assertBadParameter(utilities.validate_semantic_version(version), "Version must follow Semantic Versioning rules")

    git.change_to_branch("develop")

    git.pull("origin", "develop")

    git.delete(current)

    git.create_tag(version)


# publish
@release.command()
def publish_start():

    assertUsage(git.current_branch() == "develop", "Must publish from a develop branch")

    latest_local_version = git.get_current_local_tag()

    click.echo("Latest local version is " + latest_local_version)

    assertBadParameter(
        utilities.validate_semantic_version(latest_local_version), "Version must follow Semantic Versioning rules"
    )

    remote_versions = git.get_current_remote_tags()

    if len(remote_versions) == 0:

        click.echo("No remote versions")

    else:
        latest_remote_version = remote_versions[0]

        click.echo("Latest remote version is " + latest_remote_version)

        assertLocalVersionIsLatest(
            Version(latest_local_version) > Version(latest_remote_version),
            "Latest remote version ["
            + latest_remote_version
            + "] is higher than latest local version ["
            + latest_local_version
            + "]",
        )

    click.echo("Creating publish request")

    git.create_branch("publish")

    git.merge_request("main", "Publishing " + latest_local_version)


def validate_release_branch():
    current = git.current_branch()

    branchInfo = current.split("/")

    branchType = branchInfo[0]

    assertUsage(branchType == "release", "Branch must be release/<version>")

    version = branchInfo[1]

    assertBadParameter(utilities.validate_semantic_version(version), "Version must follow Semantic Versioning rules")

    return (current, branchType, version)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pypelinez']

package_data = \
{'': ['*']}

install_requires = \
['chuy>=1.4.0,<2.0.0',
 'click>=8.0.3,<9.0.0',
 'colorama>=0.4.4,<0.5.0',
 'semantic-version>=2.8.5,<3.0.0',
 'toml>=0.10.2,<0.11.0',
 'twine>=3.8.0,<4.0.0']

entry_points = \
{'console_scripts': ['pypelinez = pypelinez.main:main']}

setup_kwargs = {
    'name': 'pypelinez',
    'version': '0.6.1',
    'description': '',
    'long_description': '# Pypelines\n\n## Description\nPython command line tool to support CI/CD for various platforms\n\n[![pipeline status](https://gitlab.com/thecb4/pypelinez/badges/main/pipeline.svg)](https://gitlab.com/thecb4/pypelinez/-/commits/main)\n\n-- images --\n\n## Installation\npip install pypelinez\n\n## Usage\n```shell\n$ pypelinez feature start <feature/feature-name>\n```\n\nCurrent commands:\n* feature start\n* feature add-commit\n* feature submit\n* feature finish\n* release start <version>\n* release add-commit\n* release submit\n* release finish\n* release publish-start\n\n## Support\n\n## Roadmap\n[ ] apple version bump\n[ ] android version bump\n[ ] documentation\n[ ] Simplify version bump process\n\n## Contributing\n\n## Authors and Acknowledgements\n\n## License\n[MIT](https://choosealicense.com/licenses/mit/)',
    'author': 'thecb4',
    'author_email': 'cavelle@thecb4.io',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://gitlab.com/thecb4/pypelinez',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
